@@ -355,12 +355,15 @@ model.frame = function(formula, data, ...) {
 # assign should be right if input is generated by model.matrix() or is $qr slot of lm
 #' @export
 .cmpMM = function(X, weights = diag(nrow(X)), assign = attr(X$qr, "assign")) {
+    if (!get_emm_option("enable.submodel"))
+        return(NULL)
     if(!is.qr(X)) {
         if(any(is.na(X)))
             return(NULL)
-        if (!missing(weights))
-            X = suppressWarnings(diag(sqrt(weights)) %*% X)
-        X = try(qr(X), silent = TRUE)
+        X = try({
+            X = sweep(X, 1, sqrt(weights), "*") # scale rows by sqrt(weights)
+            qr(X)
+        }, silent = TRUE)
         if (inherits(X, "try-error"))
             return(NULL)
     }
