@@ -151,7 +151,7 @@ emmeans.list = function(object, specs, ...) {
 #'   are provided for further analysis, including
 #'   \code{\link{summary.emmGrid}}, \code{\link{confint.emmGrid}},
 #'   \code{\link{test.emmGrid}}, \code{\link{contrast.emmGrid}},
-#'   \code{\link{pairs.emmGrid}}, and \code{\link{CLD.emmGrid}}.
+#'   and \code{\link{pairs.emmGrid}}.
 
 #' When \code{specs} is a \code{list} or a \code{formula} having a left-hand
 #' side, the return value is an \code{\link{emm_list}} object, which is simply a
@@ -263,7 +263,8 @@ emmeans = function(object, specs, by = NULL,
                    weights, offset, trend, ..., tran) {
     
     if(!is(object, "emmGrid")) {
-        object = ref_grid(object, ...)
+        args = .zap.args(object = object, ..., omit = "submodel")
+        object = do.call(ref_grid, args)
     }
     if (is.list(specs)) {
         return (emmeans.list(object, specs, by = by, 
@@ -286,8 +287,9 @@ emmeans = function(object, specs, by = NULL,
         options $tran = tran
     }
     
-    if((length(specs) == 1) && (specs == "1"))
-        specs = character(0)
+    # This was added in 1.47, but causes problems
+    # if((length(specs) == 1) && (specs == "1"))
+    #     specs = character(0)
     
     if(is.null(nesting <- object@model.info$nesting)) 
         {
@@ -309,11 +311,12 @@ emmeans = function(object, specs, by = NULL,
         if(any(ord != seq_along(ord)))
             RG = RG[ord]
         
-        if ((length(facs) == 1) && (facs == "1")) {  ### just want grand mean
+        # xxx if ((length(facs) == 1) && (facs == "1")) {  ### just want grand mean
+        if("1" %in% facs) {
             RG@levels[["1"]] = "overall"
             RG@grid[ ,"1"] = 1
         }
-        
+
         
         # Figure out the structure of the grid
         wgt = RG@grid[[".wgt."]]
@@ -450,18 +453,11 @@ emmeans = function(object, specs, by = NULL,
     
 
     if(!missing(contr)) { # return a list with emmeans and contrasts
-        if (is.character(contr) && contr == "cld") {
-        # TO DO: provide for passing dots to cld                
-            #xxxx return(cld(result, by = by))
-            stop("`cld` in formula specs has been deprecated.\n",
-                 "To obtain a compact letter display, first call `emmeans()`,\n",
-                 "then call `CLD()` on the result.")
-        }
-        args = list(...)
-        args$data = NULL   # ensure 'data' not passed
-        args$object = result
-        args$method = contr
-        args$by = by
+        # NULL-out a bunch of arguments to not pass. 
+        dontpass = c("data", "avgd.over", "by.vars", "df", "initMesg", "estName", "estType",
+                     "famSize", "inv.lbl", "methDesc", "nesting", "pri.vars", 
+                     "tran", "tran.mult", "tran.offset", "tran2", "type", "is.new.rg")
+        args = .zap.args(object = result, method = contr, by = by, ..., omit = dontpass)
         ctrs = do.call(contrast, args)
         result = .cls.list("emm_list", emmeans = result, contrasts = ctrs)
         if(!is.null(lbl <- object@misc$methDesc))
@@ -543,7 +539,7 @@ emmeans = function(object, specs, by = NULL,
 #' ( dose.emm <- emmeans(expt.rg, "dose") )
 #' 
 #' rbind(pairs(trt.emm), pairs(dose.emm), adjust = "mvt")
-emmobj = function(bhat, V, levels, linfct, df = NA, dffun, dfargs = list(), 
+emmobj = function(bhat, V, levels, linfct = diag(length(bhat)), df = NA, dffun, dfargs = list(), 
                   post.beta = matrix(NA), ...) {
     if ((nrow(V) != ncol(V)) || (nrow(V) != ncol(linfct)) || (length(bhat) != ncol(linfct)))
         stop("bhat, V, and linfct are incompatible")
